struct VERTEX_IN
{
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 texCoord : TEXCOORD0;
};

struct VERTEX_OUT
{
	float4 pos : POSITION;
	float4 worldPos : TEXCOORD0;
	float3 normal : TEXCOORD1;
	float2 texCoord : TEXCOORD2;
	float4 shadowCoord : TEXCOORD3;
};

VERTEX_OUT main_vs(VERTEX_IN IN,
          uniform float4x4 worldMatrix,
          uniform float4x4 worldInvTr,
          uniform float4x4 worldViewProj,
          uniform float4x4 shadowMatrix)
{
	VERTEX_OUT OUT = (VERTEX_OUT)0;
	
	OUT.pos = mul(worldViewProj, IN.pos);
	OUT.texCoord = IN.texCoord;
	OUT.worldPos = mul(worldMatrix, IN.pos);
	OUT.normal = mul(worldInvTr, IN.normal).xyz;
	OUT.shadowCoord = mul(shadowMatrix, IN.pos);
	
	return OUT;
}

float4 main_ps(VERTEX_OUT IN,
       uniform float4 lightPos,
       uniform float4 lightDepthParams,
#ifdef TEXTURE_OFFSET
       uniform float4 textureOffset,
#endif // TEXTURE_OFFSET
       uniform sampler2D colorMap : register(s0),
       uniform sampler2D shadowMap : register(s1)) : COLOR0
{
	// shading
	float3 worldPos = IN.worldPos.xyz / IN.worldPos.w;
	float3 L = normalize(lightPos - worldPos);
	float3 N = normalize(IN.normal);
	float shade = max(dot(L, N), 0.0);
	
	// diffuse
#ifdef TEXTURE_OFFSET
	float2 texCoord = textureOffset.xy + IN.texCoord * 0.25;
	float4 diffuseColor = tex2D(colorMap, texCoord);
#else
	float4 diffuseColor = tex2D(colorMap, IN.texCoord);
#endif // TEXTURE_OFFSET
	
	// shadow
	float shadow = 1;
	
	if (IN.shadowCoord.z > 0)
	{
		float z = IN.shadowCoord.w / lightDepthParams.y;
		IN.shadowCoord /= IN.shadowCoord.w;
		
		float2 moments = tex2D(shadowMap, IN.shadowCoord.xy).xy;
		float lit_factor = z < moments.x + 0.001;
		
		float E_x2 = moments.y;
		float Ex_2 = moments.x * moments.x;
		float variance = min(max(E_x2 - Ex_2, 0.0) + 0.001, 1.0);
		float m_d = moments.x - z;
		float p = variance / (variance + m_d * m_d);
		
		shadow = max(lit_factor, p * 0.8);
	}
	
	return float4(shade * shadow * diffuseColor.xyz, 1.0);
}
